
$body-bg: black;
$body-color: darkgray;


$theme-colors: (
        "primary": yellow,
        "danger": orange,
        "custom-color": #900
);

$enable-rounded: true;
$enable-shadows: true;

@import "bootstrap";